/* -------------------------------------------------------------------------*
 * Samples for the Attolic True Studio and the GCC                          *
 * Copyright (C) Lucjan Bryndza <lucjan.bryndza@ep.com.pl>  19.11.2010      *
 * Example No8 for the STM32 Butterfly2 and KAMODLCD1 and KAMODTEMP         *
 * Display temp on the N3310 disp                                           *
 *                                                                          *
 * http://www.boff.pl                                                       *
 * -------------------------------------------------------------------------*/

#include "stm32f10x_lib.h"
#include "system.h"
#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>
#include "time.h"
#include "nokia_lcd.h"
#include "i2c_master.h"
#include "tiny_printf.h"

/* ----------------------------------------------------------------------- */

//RGB I2c devaddr
#define I2C_SPEED 100000
#define TEMPSENSOR_I2CADDR 0x90		/*I2c temp sensor addr */
#define MCP9800_CONFIG_REG 1		/* configuration register */
#define MCP9800_TEMP_REG 0			/* temperature register */
#define MCP9800_RES_12BIT (3<<5)	/* 12 bit resolution val */

/* ----------------------------------------------------------------------- */
//Temperature fractional structure
struct temp
{
	short t;		//temp
	short ft;		// .temp fract
};

/* ------------------------------------------------------------------ */
//Configure temperature sensor to 12 bit resolution
static int tempsensor_init(void)
{
	static const uint8_t tmp[] = { MCP9800_CONFIG_REG , MCP9800_RES_12BIT };	//Config register
	//Send configuration data
	return i2cm_transfer_7bit(TEMPSENSOR_I2CADDR,tmp,sizeof(tmp),NULL,0);
}

/* ------------------------------------------------------------------ */
//Get current temperature and fill the structure
static int tempsensor_get(struct temp *t)
{
	static const unsigned char temp_reg = MCP9800_TEMP_REG;
	static unsigned char temp[2];
	int ecode;
	//Read the temperature
	ecode = i2cm_transfer_7bit(TEMPSENSOR_I2CADDR,&temp_reg,sizeof(temp_reg),temp,sizeof(temp));
	//Convert to integer
	if(ecode>=0)
	{
		t->t = temp[0];
		t->ft = (((int)(temp[1]>>4))*10)/16;
	}
	return ecode;
}

/* ----------------------------------------------------------------------- */
//Display temperature
static void display_temp(int x,int y, struct temp *t, const char *str)
{
	char buf[16];
	tiny_snprintf( buf,sizeof(buf),"%d.%d %s", t->t, t->ft, str );
	nlcd_put_string(buf,x,y);
}

/* ----------------------------------------------------------------------- */
#define MEASURE_DELAY_TIME 200

int main(void)
{

    //Initialize system timer
    time_init();
    //Initialize nokia display
    nlcd_init();
    //Initialize the i2c module
    i2cm_init(I2C_SPEED);
    //Initialize temp controller
    int res = tempsensor_init();
    if(res<0)
    {
        char buf[24];
        tiny_snprintf( buf, sizeof(buf), "I2cErr %d",res);
        nlcd_put_string(buf,0,0);
        for(;;);    //Fail never end loop
    }
    else
    {
        nlcd_put_string( "www.boff.pl", 0, 0 );
	    nlcd_put_string( "Sensor Temp:", 0, 1 );
    }
    struct temp t;
    for(;;)
    {
        res = tempsensor_get(&t);
        if(res>=0)
        {
            display_temp( 0, 2, &t,"C      " );
        }
        else
        {
          char buf[24];
          tiny_snprintf( buf, sizeof(buf), "I2cErr %d",res);
          nlcd_put_string(buf,0,2);
          for(;;);      //Failed infinite loop
        }
        wait_ms(MEASURE_DELAY_TIME);
    }
    return 0;
}

/* ----------------------------------------------------------------------- */

